/*

  This macro read the HRes data as input file and create histograms,
  and write them to a target root file.
  The process under analysis is
  pp -> H + X -> WW + X -> l nu l nu + X
  Some experimental cuts can be applied.
  The target file is newly created and must not be identical to one
  of the source files.

  Syntax from terminal:

  ~$:root -l -q WW.C
  <input file name>

  Authors:
  Massimo Calamai, massimo.calamai@fi.infn.it
  Damiano Tommasini, tommasini@fi.infn.it
  Date:   March.8.2012

  Feel free to send comments or bug reports.

*/


#define max(a,b) ((a) > (b) ? (a) : (b))
#define min(a,b) ((a) > (b) ? (b) : (a))

#include "Riostream.h"
#include "TTree.h"
#include "TFile.h"

void WW() {

  printf("Please insert input file name:\n");
  ifstream in;
  string strtemp;
  string strtemp2;
  std::cin>>strtemp2.c_str();
  strtemp.assign("./");
  strtemp.append(strtemp2.c_str());
  in.open(strtemp2.c_str(),ios::in);
  strtemp.append(".root");
  std::cout<<" "<<strtemp.c_str()<<" \n";

  string str;
  Int_t R,V;
  Double_t wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3;
  Double_t p5_0,p5_1,p5_2,p5_3,p6_0,p6_1,p6_2,p6_3;
  Int_t i,number_plot,number_old = 0, nlines = 0;
  TFile *f = new TFile(strtemp.c_str(),"RECREATE");

  // Here we define the observables to be calculated
  Double_t eta3,eta4,eta5,eta6,pt3,pt4,pt5,pt6,cosphi45,deltaphi,pt34,pt3456,pto1,pto2,ptmiss,m45;

  // Here we define the histograms
  TH1F *eta4_hist   = new TH1F("eta4","eta4",50,0.0,5.0);
  TH1F *pt4_hist    = new TH1F("pt4","pt4",200,0.0,200.0);
  TH1F *pt34_hist   = new TH1F("pt34","pt34",100,0.0,200.0);
  TH1F *pt3456_hist = new TH1F("pt3456","pt3456",50,0.0,500.0);
  TH1F *dphi_hist   = new TH1F("deltaphi","deltaphi",18,0.0,180.0);
  TH1F *pto1_hist   = new TH1F("pto1","pto1",100,0.0,500.0);
  TH1F *pto2_hist   = new TH1F("pto2","pto2",100,0.0,500.0);

  TH1F *eta4_hist_cut   = new TH1F("eta4_cut","eta4_cut",50,0.0,5.0);
  TH1F *pt4_hist_cut    = new TH1F("pt4_cut","pt4_cut",200,0.0,200.0);
  TH1F *pt34_hist_cut   = new TH1F("pt34_cut","pt34_cut",100,0.0,200.0);
  TH1F *pt3456_hist_cut = new TH1F("pt3456_cut","pt3456_cut",50,0.0,500.0);
  TH1F *dphi_hist_cut   = new TH1F("deltaphi_cut","deltaphi_cut",18,0.0,180.0);
  TH1F *pto1_hist_cut   = new TH1F("pto1_cut","pto1_cut",100,0.0,500.0);
  TH1F *pto2_hist_cut   = new TH1F("pto2_cut","pto2_cut",100,0.0,500.0);


  // Now we start to read the file
  getline (in,str);
  getline (in,str);

  in >> V>>R;

  getline (in,str);
  getline (in,str);

  printf("\nVirt and Real iterations: %d,%d\n\n First 4 events:\n",V,R);

  // Now we start to read data and fill histograms
  while (1) {

    nlines++;
    // Here we read the data
    in >> number_plot>>wt>>p3_0>>p3_1>>p3_2>>p3_3>>p4_0>>p4_1>>p4_2>>p4_3>>p5_0>>p5_1>>p5_2>>p5_3>>p6_0>>p6_1>>p6_2>>p6_3;
    if (!in.good()) break;

    // Uncomment the following instruction for fast tests. Then you read
    // only the first 10000 events and the plots will be without physical meaning.
    //if(nlines > 10000) break;

    // You can comment/uncomment the following three if statement
    if (number_old != number_plot) {
      number_old = number_plot;
      if(number_plot>0)
        {printf("\n Virtual iteration number: %d\n",number_plot);}
      else
        {printf("\n Real iteration number: %d\n",(-number_plot));}
    }
    if (nlines < 5) printf("%d,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e,%e\n",number_plot,wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3,p5_0,p5_1,p5_2,p5_3,p6_0,p6_1,p6_2,p6_3);
    //if ((nlines%100000) == 0)printf("Analysed %d events\n",nlines);

    // Here we calculate the required variables
    eta4=0.5*log( (p4_0+p4_3)/(p4_0-p4_3) );
    eta5=0.5*log( (p5_0+p5_3)/(p5_0-p5_3) );
    pt4=sqrt( pow(p4_1,2) + pow(p4_2,2) );
    pt5=sqrt( pow(p5_1,2) + pow(p5_2,2) );
    pto1=max(pt4,pt5);
    pto2=min(pt4,pt5);
    pt34=sqrt( pow((p3_1+p4_1),2) + pow((p3_2+p4_2),2) );
    pt3456=sqrt( pow((p3_1+p4_1+p5_1+p6_1),2) + pow((p3_2+p4_2+p5_2+p6_2),2) );
    cosphi45=( p4_1*p5_1+p4_2*p5_2 ) / ( pt4*pt5 );
    // We avoid problems of rounding in the function acos
    if(cosphi45>1.){
      printf("Problem cosphi45: %e\n",cosphi45);
      cosphi45=1.;
    }
    if(cosphi45<-1.){
      printf("Problem cosphi45: %e\n",cosphi45);
      cosphi45=-1.;
    }
    deltaphi=fabs(acos(cosphi45))*57.2957795130823; // 57.2957795130823=180./3.141592653589793
    // We avoid problems of rounding on deltaphi
    if(deltaphi>180.){
      printf("Problem deltaphi: %e\n",deltaphi);
      deltaphi=180.;
    }
    if(deltaphi<0.){
      printf("Problem deltaphi: %e\n",deltaphi);
      deltaphi=0.;
    }
    ptmiss=sqrt( pow((p3_1+p6_1),2) + pow((p3_2+p6_2),2) );
    m45=sqrt( pow((p4_0+p5_0),2) - pow((p4_1+p5_1),2) - pow((p4_2+p5_2),2) -pow((p4_3+p5_3),2) );


    // Here we fill the histograms

    eta4_hist->Fill(eta4,wt);
    pt4_hist->Fill(pt4,wt);
    pt34_hist->Fill(pt34,wt);
    pt3456_hist->Fill(pt3456,wt);
    dphi_hist->Fill(deltaphi,wt);
    pto1_hist->Fill(pto1,wt);
    pto2_hist->Fill(pto2,wt);


    // Here we fill the histograms only if the event pass the cut selection
    if (ptmiss>30.0 && m45>12. && pto2>20. && fabs(eta4)<2.5 && fabs(eta5)<2.5) {
      eta4_hist_cut->Fill(eta4,wt);
      pt4_hist_cut->Fill(pt4,wt);
      pt34_hist_cut->Fill(pt34,wt);
      pt3456_hist_cut->Fill(pt3456,wt);
      dphi_hist_cut->Fill(deltaphi,wt);
      pto1_hist_cut->Fill(pto1,wt);
      pto2_hist_cut->Fill(pto2,wt);
    }

  }

  // Root provides the number of events for each bin.
  // Here we normalise the histogram to obtain the correct physical normalisation
  // as in the topdrawer file. The histogram must be scaled as 1/binsize
  eta4_hist->Scale(1./(eta4_hist->GetBinWidth(1)));
  pt4_hist->Scale(1./(pt4_hist->GetBinWidth(1)));
  pt34_hist->Scale(1./(pt34_hist->GetBinWidth(1)));
  pt3456_hist->Scale(1./(pt3456_hist->GetBinWidth(1)));
  dphi_hist->Scale(1./(dphi_hist->GetBinWidth(1)));
  pto1_hist->Scale(1./(pto1_hist->GetBinWidth(1)));
  pto2_hist->Scale(1./(pto2_hist->GetBinWidth(1)));

  eta4_hist_cut->Scale(1./(eta4_hist_cut->GetBinWidth(1)));
  pt4_hist_cut->Scale(1./(pt4_hist_cut->GetBinWidth(1)));
  pt34_hist_cut->Scale(1./(pt34_hist_cut->GetBinWidth(1)));
  pt3456_hist_cut->Scale(1./(pt3456_hist_cut->GetBinWidth(1)));
  dphi_hist_cut->Scale(1./(dphi_hist_cut->GetBinWidth(1)));
  pto1_hist_cut->Scale(1./(pto1_hist_cut->GetBinWidth(1)));
  pto2_hist_cut->Scale(1./(pto2_hist_cut->GetBinWidth(1)));


  // Here we close the histograms and the routine
  eta4_hist->Write();
  pt4_hist->Write();
  pt34_hist->Write();
  pt3456_hist->Write();
  dphi_hist->Write();
  pto1_hist->Write();
  pto2_hist->Write();

  eta4_hist_cut->Write();
  pt4_hist_cut->Write();
  pt34_hist_cut->Write();
  pt3456_hist_cut->Write();
  dphi_hist_cut->Write();
  pto1_hist_cut->Write();
  pto2_hist_cut->Write();

  nlines--;
  printf("Found %d events\n",nlines);
  in.close();
  delete f;
}
