/*

  This macro read the HRes data as input file and create histograms,
  and write them to a target root file.
  The process under analysis is
  pp -> H + X -> gamma gamma + X
  Some experimental cuts can be applied.
  The target file is newly created and must not be identical to one
  of the source files.

  Syntax from terminal:

  ~$:root -l -q gamgam.C
  <input file name>

  Authors:
  Massimo Calamai, massimo.calamai@fi.infn.it
  Damiano Tommasini, tommasini@fi.infn.it
  Date:   March.8.2012

  Feel free to send comments or bug reports.

*/


#define max(a,b) ((a) > (b) ? (a) : (b))
#define min(a,b) ((a) > (b) ? (b) : (a))

#include "Riostream.h"
#include "TTree.h"
#include "TFile.h"

void gamgam() {

  printf("Please insert input file name:\n");
  ifstream in;
  string strtemp;
  string strtemp2;
  std::cin>>strtemp2.c_str();
  strtemp.assign("./");
  strtemp.append(strtemp2.c_str());
  in.open(strtemp2.c_str(),ios::in);
  strtemp.append(".root");
  std::cout<<" "<<strtemp.c_str()<<" \n";

  string str;
  Int_t R,V;
  Double_t wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3;
  Int_t i,number_plot, number_old = 0, nlines = 0;
  TFile *f = new TFile(strtemp.c_str(),"RECREATE");

  // Here we define the observables to be calculated
  Double_t eta3,eta4,pt3,pt4,ptmin,ptmax,y34,pt34,norm,pTt;

  // Here we define the histograms
  TH1F *eta4_hist  = new TH1F("eta4","eta4",100,0.0,5.0);
  TH1F *pt4_hist   = new TH1F("pt4","pt4",100,0.0,200.0);
  TH1F *pt34_hist  = new TH1F("pt34","pt34",100,0.0,200.0);
  TH1F *ptmin_hist = new TH1F("ptmin","ptmin",250,0.0,500.0);
  TH1F *ptmax_hist = new TH1F("ptmax","ptmax",250,0.0,500.0);

  TH1F *eta4_hist_cut  = new TH1F("eta4 cut","eta4 cut",60,0.0,3.0);
  TH1F *pt4_hist_cut   = new TH1F("pt4 cut","pt4 cut",100,0.0,200.0);
  TH1F *pt34_hist_cut  = new TH1F("pt34 cut","pt34 cut",100,0.0,200.0);
  TH1F *ptmin_hist_cut = new TH1F("ptmin cut","ptmin cut",250,0.0,500.0);
  TH1F *ptmax_hist_cut = new TH1F("ptmax cut","ptmax cut",250,0.0,500.0);
  TH1F *y34_hist_cut   = new TH1F("y34 cut","y34 cut",100,0.0,5.0);
  TH1F *pTt_hist_cut   = new TH1F("pTt cut","pTt cut",200,0.0,400.0);

  TH1F *eta4_hist_cut_bis = new TH1F("eta4 cut_bis","eta4 cut_bis",60,0.0,3.0);
  TH1F *pt4_hist_cut_bis  = new TH1F("pt4 cut_bis","pt4 cut_bis",100,0.0,200.0);

  // Now we start to read the file
  getline (in,str);
  getline (in,str);

  in >> V>>R;

  getline (in,str);
  getline (in,str);

  printf("\nVirt and Real iterations: %d,%d\n\n First 4 events:\n",V,R);

  // Now we start to read data and fill histograms
  while (1) {
    nlines++;
    // Here we read the data
    in >> number_plot>>wt>>p3_0>>p3_1>>p3_2>>p3_3>>p4_0>>p4_1>>p4_2>>p4_3;
    if (!in.good()) break;

    // Uncomment the following instruction for fast tests. Then you read
    // only the first 10000 events and the plots will be without physical meaning.
    //if(nlines > 10000) break;

    // You can comment/uncomment the following three if statement
    if (number_old != number_plot) {
      number_old = number_plot;
      if(number_plot>0)
        {printf("\n Virtual iteration number: %d\n",number_plot);}
      else
        {printf("\n Real iteration number: %d\n",(-number_plot));}
    }
    if (nlines < 5) printf("%d,%e,%e,%e,%e,%e,%e,%e,%e,%e\n",number_plot,wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3);
    //if ((nlines%100000) == 0)printf("Analysed %d events\n",nlines);

    // Here we calculate the required variables
    eta3=0.5*log( (p3_0+p3_3)/(p3_0-p3_3) );
    eta4=0.5*log( (p4_0+p4_3)/(p4_0-p4_3) );
    pt3=sqrt( pow(p3_1,2) + pow(p3_2,2) );
    pt4=sqrt( pow(p4_1,2) + pow(p4_2,2) );
    ptmin=min(pt3,pt4);
    ptmax=max(pt3,pt4);
    pt34=sqrt( pow((p3_1+p4_1),2) + pow((p3_2+p4_2),2) );
    norm=sqrt( pow((p3_1-p4_1),2) + pow((p3_2-p4_2),2) );
    pTt=fabs( ( (p3_1+p4_1)*(p3_2-p4_2) - (p3_2+p4_2)*(p3_1-p4_1) ) /norm );

    // Here we fill the histograms
    eta4_hist->Fill(eta4,wt);
    pt4_hist->Fill(pt4,wt);
    pt34_hist->Fill(pt34,wt);
    ptmin_hist->Fill(ptmin,wt);
    ptmax_hist->Fill(ptmax,wt);

    // Here we fill the histograms only if the event pass the cut selection
    if (fabs(eta3)<2.5 && fabs(eta4)<2.5 && ptmin>25.0 && ptmax>40.0) {
      eta4_hist_cut->Fill(eta4,wt);
      pt4_hist_cut->Fill(pt4,wt);
      pt34_hist_cut->Fill(pt34,wt);
      ptmin_hist_cut->Fill(ptmin,wt);
      ptmax_hist_cut->Fill(ptmax,wt);
      pTt_hist_cut->Fill(pTt,wt);
    }

     // We can also fill other histograms by different cut selection
    if (fabs(eta3)<2. && fabs(eta4)<2. && pt3>30.0 && pt4>30.0) {
      eta4_hist_cut_bis->Fill(eta4,wt);
      pt4_hist_cut_bis->Fill(pt4,wt);
    }
  }

  // Root provides the number of events for each bin.
  // Here we normalise the histogram to obtain the correct physical normalisation
  // as in the topdrawer file. The histogram must be scaled as 1/binsize
  eta4_hist->Scale(1./(eta4_hist->GetBinWidth(1)));
  pt4_hist->Scale(1./(pt4_hist->GetBinWidth(1)));
  pt34_hist->Scale(1./(pt34_hist->GetBinWidth(1)));
  ptmin_hist->Scale(1./(ptmin_hist->GetBinWidth(1)));
  ptmax_hist->Scale(1./(ptmax_hist->GetBinWidth(1)));

  eta4_hist_cut->Scale(1./(eta4_hist_cut->GetBinWidth(1)));
  pt4_hist_cut->Scale(1./(pt4_hist_cut->GetBinWidth(1)));
  pt34_hist_cut->Scale(1./(pt34_hist_cut->GetBinWidth(1)));
  ptmin_hist_cut->Scale(1./(ptmin_hist_cut->GetBinWidth(1)));
  ptmax_hist_cut->Scale(1./(ptmax_hist_cut->GetBinWidth(1)));
  pTt_hist_cut->Scale(1./(pTt_hist_cut->GetBinWidth(1)));

  eta4_hist_cut_bis->Scale(1./(eta4_hist_cut_bis->GetBinWidth(1)));
  pt4_hist_cut_bis->Scale(1./(pt4_hist_cut_bis->GetBinWidth(1)));


  // Here we close the histograms and the routine
  eta4_hist->Write();
  pt4_hist->Write();
  pt34_hist->Write();
  ptmin_hist->Write();
  ptmax_hist->Write();

  eta4_hist_cut->Write();
  pt4_hist_cut->Write();
  pt34_hist_cut->Write();
  ptmin_hist_cut->Write();
  ptmax_hist_cut->Write();
  pTt_hist_cut->Write();

  eta4_hist_cut_bis->Write();
  pt4_hist_cut_bis->Write();

  nlines--;
  printf("Found %d events\n",nlines);
  in.close();
  delete f;
}
