/*

  This macro read the HRes data as input file and create histograms (and tree),
  and write them to a target root file.

  In particular a correct errors evaluation is implemented. Indeed ROOT
  error evaluation for histrograms is based on Poisson distribution.

  The process under analysis is
  pp -> H + X -> gamma gamma + X

  The target file is newly created and must not be identical to one
  of the source files. It share most of the filename of the data file
  and is created on the same relative-path directory.

  Syntax:

  ~$:root -l -q gamgam.C
  <input file name>

  Author: Massimo Calamai massimo.calamai@fi.infn.it
  Date:   March.18.2012

  I have tested this macro on ASCII file. Feel free to send comments
  or bug reports.

*/



#include "Riostream.h"
#include "TTree.h"
#include "TFile.h"
const Int_t Rdim=1000;
const Int_t Vdim=1000;


void gamgam_error() {
  Int_t R,V;
  printf("Please insert input file name:\n");
  ifstream in;
  string strtemp;
  string strtemp2;
  char strname[100],strtitle[100];
  std::cin>>strtemp2.c_str();
  strtemp.assign("./");
  strtemp.append(strtemp2.c_str());
  in.open(strtemp2.c_str(),ios::in);
  strtemp.append("_error.root");
  std::cout<<" "<<strtemp.c_str()<<" \n";


  string str;
  Double_t dump,wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3;
  Double_t sigma=0,mean=0;
  Int_t i,number_plot,number_old = 0, nlines = 0;
  TFile *f = new TFile(strtemp.c_str(),"RECREATE");
  //  TTree *T = new TTree("T","full total trees");// uncomment if you want a tree file
  //  T->Branch("Event",&nlines,"event/I");         // uncomment if you want a tree file
  //  T->Branch("number_plot",&number_plot,"number_plot/I");// uncomment if you want a tree file
  //  T->Branch("wt",&wt,"wt/D");// uncomment if you want a tree file
  //  T->Branch("p3_0",&p3_0,"p3_0/D");// uncomment if you want a tree file
  //  T->Branch("p3_1",&p3_1,"p3_1/D");// uncomment if you want a tree file
  //  T->Branch("p3_2",&p3_2,"p3_2/D");// uncomment if you want a tree file
  //  T->Branch("p3_3",&p3_3,"p3_3/D");// uncomment if you want a tree file
  //  T->Branch("p4_0",&p4_0,"p4_0/D");// uncomment if you want a tree file
  //  T->Branch("p4_1",&p4_1,"p4_1/D");// uncomment if you want a tree file
  //  T->Branch("p4_2",&p4_2,"p4_2/D");// uncomment if you want a tree file
  //  T->Branch("p4_3",&p4_3,"p4_3/D");// uncomment if you want a tree file
  TH1F  *pt34R[Rdim];
  TH1F  *pt34V[Vdim];



  //Here we call the histogram for the Higgs boson transverse momentum
  TH1F *pt34 = new TH1F("pt34_distr","pt34  distribution",100,0,200.0);


  //Here we call the final Vegas histograms for the real and virtual part
  //ave-histogram contains the right multiplicity of events but the wrong ROOT default errors
  TH1F *pt34R_ave = new TH1F("pt34_real_ave","pt34 real ave distribution",100,0.0,200.0);
  TH1F *pt34V_ave = new TH1F("pt34_virt_ave","pt34 virt ave distribution",100,0,200.0);

  //Here we call the final Vegas histograms for the real and virtual part
  //mean-histogram contains the right errors calculation
  TH1F *pt34R_mean = new TH1F("pt34_real_mean","pt34 real mean distribution",100,0.0,200.0);
  TH1F *pt34V_mean = new TH1F("pt34_virt_mean","pt34 virt mean distribution",100,0.0,200.0);



  //Here we call the all real Vegas histograms
  for (i=0;i<Rdim;i++){
    std::sprintf(strname,"real_plot_number_%d",i+1);
    std::sprintf(strtitle,"Distrib  real %d",i+1);
    pt34R[i] = new TH1F(strname,strtitle,100,0.0,200.0);
  }
  //Here we call the all real Vegas histograms
  for (i=0;i<Vdim;i++){
    std::sprintf(strname,"virtual_plot_number_%d",i+1);
    std::sprintf(strtitle,"Distrib virt %d",i+1);
    pt34V[i] = new TH1F(strname,strtitle,100,0.0,200.0);
  }


  getline (in,str);
  getline (in,str);
  in >> V>>R;

  getline (in,str);
  getline (in,str);

  if(V > Vdim || R > Rdim){
    printf(" ERROR!\n V=%d or R=%d too big\n",R,V);
    return 0;
  }

  printf("\nVirt and Real iterations: %d,%d\n\n First 4 events:\n",V,R);

  while (1) {
    nlines++;
    // Here we read the data
    in >> number_plot>>wt>>p3_0>>p3_1>>p3_2>>p3_3>>p4_0>>p4_1>>p4_2>>p4_3;
    if (!in.good()) break;

    // You can comment/uncomment the following if statements
    if (number_old != number_plot) {
      number_old = number_plot;
      if(number_plot>0)
        {printf("\n Virtual iteration number: %d\n",number_plot);}
      else
        {printf("\n Real iteration number: %d\n",(-number_plot));}
    }
    if (nlines < 5) printf("%d,%e,%e,%e,%e,%e,%e,%e,%e,%e\n",number_plot,wt,p3_0,p3_1,p3_2,p3_3,p4_0,p4_1,p4_2,p4_3);

    // Here we calculate the Higgs boson transverse momentum
    dump=sqrt( pow((p3_1+p4_1),2) + pow((p3_2+p4_2),2) );

    // Here we create a different plot for every Vegas iteration
    if(number_plot>0) {
      pt34V[number_plot-1]->Fill(dump,wt);
    }
    else {
      pt34R[abs(number_plot)-1]->Fill(dump,wt);
    }
  }




  //Here we ask root to take into account errors evaluation for final-step histograms
  pt34->Sumw2();
  pt34R_ave->Sumw2();
  pt34V_ave->Sumw2();


  //Here we create ave-histrograms those contains the right multiplicity
  for (i=0;i<R;i++){
    pt34R_ave->Add(pt34R[i]);
  }
  for (i=0;i<V;i++){
    pt34V_ave->Add(pt34V[i]);
  }


  /* ******      OUR  ERROR IMPLEMENTATION  START                           ****   */
  // Here we calculate the mean value for each bin
  for (Int_t j=0; j<(pt34R_mean->GetNbinsX()+1);j++){
    mean=0;
    for (i=0;i<R;i++){
      mean=mean+pt34R[i]->GetBinContent(j);
    }
    pt34R_mean->SetBinContent(j,mean);
  }



  for (Int_t j=0; j<(pt34V_mean->GetNbinsX()+1);j++){
    mean=0;
    for (i=0;i<V;i++){
      mean=mean+pt34V[i]->GetBinContent(j);
    }
    pt34V_mean->SetBinContent(j,mean);
  }


  // Here we estimate the error for each bin

  for (Int_t j=0; j<(pt34R_mean->GetNbinsX()+1);j++){
    sigma=0;
    for (i=0;i<R;i++){
      dump=pt34R[i]->GetBinContent(j)*R-pt34R_mean->GetBinContent(j);
      sigma=sigma+pow(dump,2);
    }
    sigma=sqrt(sigma)/(R-1.0);
    pt34R_ave->SetBinError(j,sigma);
    pt34R_mean->SetBinError(j,sigma);
  }

  for (Int_t j=0; j<(pt34V_mean->GetNbinsX()+1);j++){
    sigma=0;
    for (i=0;i<V;i++){
      dump=pt34V[i]->GetBinContent(j)*V-pt34V_mean->GetBinContent(j);
      sigma=sigma+pow(dump,2);
    }
    sigma=sqrt(sigma)/(V-1.0);
    pt34V_ave->SetBinError(j,sigma);
    pt34V_mean->SetBinError(j,sigma);
  }
  /* ******  END OF OUR  ERROR IMPLEMENTATION                           ****   */



  //Here we create the plot for Higgs boson transverse momentum with statistical errors
  pt34->Add(pt34V_ave);
  pt34->Add(pt34R_ave);


  // Root provides the number of events for each bin.
  // Here we normalise the histogram to obtain the correct physical normalisation
  // as in the topdrawer file. The histogram must be scaled as 1/binsize
  // and the total number of iterations
  pt34V_mean->Scale(1./(pt34V_mean->GetBinWidth(1)));
  pt34R_mean->Scale(1./(pt34R_mean->GetBinWidth(1)));
  pt34V_ave->Scale(1./(pt34V_ave->GetBinWidth(1)));
  pt34R_ave->Scale(1./(pt34R_ave->GetBinWidth(1)));

  pt34->Scale(1./(pt34->GetBinWidth(1)));

  // If you want, you can save the interim plots
  //pt34V_mean->Write();
  //pt34R_mean->Write();
  //pt34V_ave->Write();
  //pt34R_ave->Write();
  //for (i=0;i<R;i++){
  //  pt34R[i]->Write();
  //}
  //for (i=0;i<V;i++){
  //  pt34V[i]->Write();
  //}

  // This is the final plot with the error estimation
  pt34->Write();


  nlines--;
  printf("Found %d events\n",nlines);
  in.close();
  //T->Print(); // uncomment if you want print tree file info
  //T->Write(); // uncomment if you want to save a tree file
  delete f;
}
