! Interface for virt-integration for the Higgs boson production (resummed component) and decay in 2photons or 4leptons
! gg -> H + X -> 2gamma + X
! gg -> H + X -> WW + X -> 4leptons + X
! gg -> H + X -> ZZ + X -> 4leptons + X
! X is the resummed soft-gluon emission

! NB convention on the 4-vectrs: spatial-indices 1,2,3; time/enery-index 4
! we use: 1=x,2=y (x,y transverse plane); 3=z (beam pipe); 4=t
! qt= Higgs transverse momentum, y= rapidity, mhiggs= on/off-shell Higgs mass

      double precision function lowres(r,wgt)
      implicit none
       include 'constants.f'
       include 'masses.f'
       include 'vegas_common.f'
       include 'removebr.f'
       include 'limits.f'
       include 'zerowidth.f'
       include 'vzerowidth.f'
       include 'ewcouple.f'
       include 'qcdcouple.f'
      integer higgsdec,ndec,calcola,approxim,j,k
      integer ii,kk,jj,numdecad,halfnumdecad,nwidth
      double precision random(mxdim),wt,alpha,beta,ran2
      double precision r(mxdim),sqrts,resummed,subtraction,BrnRat,
     .   gamgambr,br0,sigma0,factor0,ratioitmx,resummedt,resummedb
      double precision ppp(6,4,40000),event(40000),wtevent(40000)
      double precision eventb(40000)
      double precision jac,mhiggs,m2,qt,s,cosh2y,cutoff
      double precision mHmin,mHmax,m2min,m2max,val,wgt,p(mxpart,4)
     .   ,pH(4),p3cm(4)
     .   ,ptilde(mxpart,4),pHtilde(4),p3tilde(4),p4tilde(4),p5tilde(4)
     .   ,p6tilde(4),valtb
      double precision p3(4),p4(4),p5(4),p6(4),p9(4),p10(4)
      double precision y,ymax,phi,xx1,xx2,xxtemp,dexpy,dexpmy
      logical bin,cuts
      double precision mia,xmb,xmt,as0
      double precision msqc(-nf:nf,-nf:nf),msqctilde(-nf:nf,-nf:nf)
     .   ,msqcmedio ,msqcmediotilde,wtmedio
      double precision msqct(-nf:nf,-nf:nf),msqctildet(-nf:nf,-nf:nf)
     .   ,msqcmediot ,msqcmediotildet,wtmediot
      double precision msqcb(-nf:nf,-nf:nf),msqctildeb(-nf:nf,-nf:nf)
     .   ,msqcmediob ,msqcmediotildeb,wtmediob
       common/ratioitmx/ratioitmx
       common/nwidth/nwidth
      double precision qtminvirt,qtminreal,qtmax1,qtmax2,qtsw
       common/qtcut/qtminvirt,qtminreal,qtmax1,qtmax2,qtsw
       common/bin/bin
       common/mlimits/mHmin,mHmax
       common/energy/sqrts
       common/higgsdec/higgsdec,ndec
       common/xm/xmb,xmt
       common/BrnRat/BrnRat
       common/gamgambr/gamgambr,br0
       double precision resummscale,resummscaleb
       common/resummscale/resummscale,resummscaleb
       double precision mtop,mbot,sigmatb,sigmat
CH
       common/flag/approxim
       common/Born/sigmatb,sigmat
CH
       common/as0/as0

      s=sqrts**2

! Variables for the decays loop, numdecad MUST be even; numdecad <= 40000
      numdecad=10000
      if(higgsdec.eq.1) numdecad=1000
      halfnumdecad=numdecad/2+1
      calcola=0
      msqcmedio=0d0
      wtmedio=0d0
      msqcmediot=0d0
      wtmediot=0d0
      msqcmediob=0d0
      wtmediob=0d0


! To improve the integration speed, we use Breit-Wigner density
! jac=Jacobian
      m2min=mHmin**2
      m2max=mHmax**2
      call breitw(r(1),m2min,m2max,hmass,hwidth,m2,wt)
      jac=wt
      mhiggs=dsqrt(m2)


! Rapidity extraction, we assume symmetric rapidity to obtain a double statistic
! Kinematical ymax:

      ymax=0.5d0*dlog(s/m2)
!      y=-ymax+2d0*ymax*r(2)
      y=ymax*r(2)
      jac=jac*ymax

      dexpy=dexp(y)
      dexpmy=dexp(-y)
      cosh2y=((dexpy+dexpmy)*0.5d0)**2


! qt extraction

      qt=(qtmax2-qtminvirt)*r(3)-qtminvirt
      jac=jac*(qtmax2-qtminvirt)
      if(qt.gt.qtmax1 .and. qt.lt.qtmax2)then
        jac=jac*(dcos((qt-qtmax1)*pi/qtsw)+1d0)*0.5d0
      endif


! Angular extraction
      phi=twopi*ran2()



! Now, decady kinematics
!      xx1=dsqrt(m2/s)*dexpy
!      xx2=dsqrt(m2/s)*dexpmy

! We calculate the 4-momentaof incoming partons 1 and 2 (massless)
      p(1,1)=0d0
      p(1,2)=0d0
      p(1,3)=mhiggs*dexpy*0.5d0
      p(1,4)=p(1,3)
      p(2,1)=0d0
      p(2,2)=0d0
      p(2,3)=-mhiggs*dexpmy*0.5d0
      p(2,4)=-p(2,3)

      do jj=1,4
        do ii=5,mxpart
          p(ii,jj)=0d0
        enddo
      enddo


! Higgs boson 4-momentum, with constraint: ph(4)^2-ph(1)^2-ph(2)^2-ph(3)^2=m^2
      ph(1)=qt*dcos(phi)
      ph(2)=qt*dsin(phi)
      ph(3)=0.5d0*(dexpy-dexpmy)*dsqrt(m2+qt*qt)
      ph(4)=ph(3)*(dexpy+dexpmy)/(dexpy-dexpmy)
! Pseudo-Event: Higgs with qt=0, but same m^2 and rapidity
      phtilde(1)=0d0
      phtilde(2)=0d0
!      phtilde(3)=ph(3)
!      phtilde(4)=ph(4)
      phtilde(3)=0.5d0*(dexpy-dexpmy)*mhiggs
      phtilde(4)=phtilde(3)*(dexpy+dexpmy)/(dexpy-dexpmy)


!*************************************************************************************
! Loop on Higgs boson decays
!*************************************************************************************

      do jj=1,numdecad

!Here we decide how the choose the Higgs rapidity: random or deterministically, but in any case 50%
! Random:
!       if(ran2().le.0.5d0) then
! Determinist
       if(jj.eq.halfnumdecad) then
         y=-y
         xxtemp=dexpmy
         dexpmy=dexpy
         dexpy=xxtemp
         p(1,3)=mhiggs*dexpy*0.5d0
         p(1,4)=p(1,3)
         p(2,3)=-mhiggs*dexpmy*0.5d0
         p(2,4)=-p(2,3)
         ph(3)=-ph(3)
         phtilde(3)=-phtilde(3)
       endif


! Here we generate the decays and we calculate the matrix elements
! If we use r() we are integrating with vegas, if ran2() we average on the respective variables

!***higgsdec****H->2gamma or H->(f,fbar)********************************************
      if(higgsdec.eq.1) then

!        random(1)=r(5)
!        random(2)=r(6)
        random(1)=ran2()
        random(2)=ran2()

        call phi3m0(random(1),random(2),phtilde,p3,p4,wt,*89)
!        call phi3m0(random(1),random(2),ph,p3,p4,wt,*88)
        do ii=1,4
          p(3,ii)=p3(ii)
          p(4,ii)=p4(ii)
        enddo

! We can use the bb-bar or 2gamma matrix element, and then correct with the right BrnRat
!        call gg_hgamgam(p,msqc)
CH
        call gg_h(p,msqc)   
      
CH

        call phi3m0(random(1),random(2),ph,p3,p4,wt,*88)
        do ii=1,4
          p(3,ii)=p3(ii)
          p(4,ii)=p4(ii)
        enddo

! NB: wt=1/8/pi in phi3m0

        wtevent(jj)=wt

!***higgsdec****H->VV->4l where VV=(W+,W-) or VV=(Z0,Z0)****************************
      else

        do ii=1,8
!          kk=ii+4
!          random(ii)=r(kk)
          random(ii)=ran2()
        enddo

        call phi4(random,phtilde,p3,p4,p5,p6,wt,*99)
!        call phi4(random,ph,p3,p4,p5,p6,wt,*98)
        do ii=1,4
          p(3,ii)=p3(ii)
          p(4,ii)=p4(ii)
          p(5,ii)=p5(ii)
          p(6,ii)=p6(ii)
        enddo

        if(higgsdec.eq.2) then
         call qqb_hww(p,msqc)
        else
         call qqb_hzz(p,msqc)
        endif

        call phi4(random,ph,p3,p4,p5,p6,wt,*98)

        do ii=1,4
          p(3,ii)=p3(ii)
          p(4,ii)=p4(ii)
          p(5,ii)=p5(ii)
          p(6,ii)=p6(ii)
        enddo


        wtevent(jj)=wt

!***higgsdec****end**********************************************************************
       endif

! All the full events are stored in ppp(ii,kk,jj)
         do ii=1,6,1
           do kk=1,4,1
             ppp(ii,kk,jj)=p(ii,kk)
           enddo
         enddo


! Normalization factor, inverse of Born cross section times flux factor
! NB:    Born=aass**2 * mhiggs**2 /pi/576d0/ vevsq

!        factor0=aass**2 * hmass**2 * mhiggs**2 /576d0/pi/ vevsq
        factor0=as0**2 * mhiggs**4 /576d0/pi/ vevsq


       if (.not.cuts(p,0)) then
         calcola=calcola+1
         event(jj)=msqc(0,0)/factor0/brnrat
         eventb(jj)=msqc(0,0)/factor0/brnrat
         msqcmedio=msqcmedio+event(jj)*wtevent(jj)
       else
         event(jj)=0d0
         eventb(jj)=0d0
       endif

!*************************************************************************************
      enddo ! end loop on numdecad
!*************************************************************************************


! If no event pass the cuts, go to exit
      if(calcola.eq.0) goto 199

! Fast test function "resumm", same integral for Mathematica:
!s = 8000^2;
!mhiggs = 125;
!m2 = mhiggs^2;
!cutoff = 200;
!ymax = (1/2)*Log[s/m2];
!resumm := (100/ymax)*qt*Exp[-0.1*qt]*(Cos[y*Pi/ymax] + 1)/2;
!N[Integrate[resumm, {qt, 0.5, cutoff}, {y, -ymax, ymax}]]
!N[Integrate[resumm, {qt, 0., cutoff}, {y, -ymax, ymax}]]

!       resummed=(1d2*/ymax)*qt*dexp(-0.1d0*qt)*
!     .        (dCos(y*Pi/ymax) + 1d0)/2d0 

! True resumm function:

CH     
         if(approxim.eq.2)then
          call resumm(resummedt,subtraction,mhiggs,y,qt,0)
! Sometimes the interatrion routine fails
          IF(resummedt.NE.resummedt) then
           write(*,*)'WARNING:resummedt=',resummedt,'in qt=',qt,'y=',y
           resummedt=0d0
          endif
          call resumm(resummedb,subtraction,mhiggs,y,qt,1)
! Sometimes the interatrion routine fails
          IF(resummedb.NE.resummedb) then
           write(*,*)'WARNING:resummedb=',resummedb,'in qt=',qt,'y=',y
           resummedb=0d0
          endif
          resummed=resummedt+resummedb
         else
          call resumm(resummed,subtraction,mhiggs,y,qt,0)
          IF(resummed.NE.resummed) then
           write(*,*)'WARNING: resummed=',resummed,'in qt=',qt,'y=',y
           resummed=0d0
          endif
         endif
       
CH

        


! Risults for vegas and plotter
        msqcmedio=msqcmedio/dfloat(numdecad)
        lowres=jac*resummed*msqcmedio
        val=jac*resummed*wgt/dfloat(itmx)/dfloat(numdecad)
        valtb=jac*resummedb*wgt/dfloat(itmx)/dfloat(numdecad)
         

! This weight comes from the change of the variable virtmyitmx in integration.f and plotter.f
!        if(mypart.eq.'virt') val=val*ratioitmx

        if (bin) then
        do jj=1,numdecad
           event(jj)=wtevent(jj)*event(jj)*val
! Resume event from ppp
           do ii=1,6,1
           do kk=1,4,1
             p(ii,kk)=ppp(ii,kk,jj)
           enddo
           enddo
           call plotter(p,event(jj),0)
        enddo
        endif
        


!       write(*,*)mhiggs,qt,y,resummed,subtraction,lowres

      RETURN



! Exit in case of event not accepted by cut selection
 199  continue
      lowres=0d0
      RETURN


! Exit in case of error
! Uscite non troppo corrette perche' pesa un evento
! ed esce senza completare il ciclo sui decadimenti numdecad
! In realta non dovrebbe mai passarci

 88   continue
       write(*,*)'WARNING RESUMMATION!!!! phi3m0'
!       write(*,*)mhiggs,p
       lowres=0d0
       RETURN

 98    continue 
       write(*,*)'WARNING RESUMMATION!!!! phi4'
!       write(*,*)mhiggs,p
       lowres=0d0
       RETURN

 89    continue
       write(*,*)'WARNING RESUMMATION!!!! phi3m0tilde'
!       write(*,*)mhiggs,p
       lowres=0d0
       RETURN

 99    continue
       write(*,*)'WARNING RESUMMATION!!!! phi4tilde'
!       write(*,*)mhiggs,p
       lowres=0d0
       RETURN



       END

