      DOUBLE PRECISION FUNCTION ALPHAS(Q,AMZ,NLOOP)
c     Evaluation of strong coupling constant alpha_S
c     Author: R.K. Ellis
!     Modified by D. Tommasini 24/10/2011

c     q -- scale at which alpha_s is to be evaluated
c     amz -- value of alpha_s at the mass of the Z-boson
c     nloop -- the number of loops (1,2, or 3) at which beta 
c     function is evaluated to determine running.
c     the values of the cmass and the bmass should be set
c     in common block qmass.

      IMPLICIT NONE
      DOUBLE PRECISION Q,T,AMZ,AMZ0,AMB,AMC,ZMASS,BMASS,CMASS,AS_OUT
      include 'ewinput.f'
      INTEGER NLOOP,NLOOP0,NF3,NF4,NF5
      PARAMETER(NF5=5,NF4=4,NF3=3)
      COMMON/QMASS/CMASS,BMASS
      SAVE AMZ0,NLOOP0,AMB,AMC
      DATA AMZ0,NLOOP0/0D0,0/
      
      zmass=zmass_inp
      IF (Q .LE. 0D0) THEN 
         WRITE(6,*) 'q .le. 0 in alphas'
         WRITE(6,*) 'q= ',Q
         stop
      ENDIF
      IF (AMZ .LE. 0D0) THEN 
         WRITE(6,*) 'amz .le. 0 in alphas',AMZ
         stop
      ENDIF
      IF (CMASS .LE. 0.3D0) THEN 
         WRITE(6,*) 'cmass .le. 0.3GeV in alphas',CMASS
         WRITE(6,*) 'COMMON/QMASS/CMASS,BMASS'
         stop
      ENDIF
      IF (BMASS .LE. 0D0) THEN 
         WRITE(6,*) 'bmass .le. 0 in alphas',BMASS
         WRITE(6,*) 'COMMON/QMASS/CMASS,BMASS'
         stop
      ENDIF
c--- establish value of coupling at b- and c-mass and save
      IF ((AMZ .NE. AMZ0) .OR. (NLOOP .NE. NLOOP0)) THEN
         AMZ0=AMZ
         NLOOP0=NLOOP
         T=2D0*DLOG(BMASS/ZMASS)
         CALL NEWTON1(T,AMZ,AMB,NLOOP,NF5)
         T=2D0*DLOG(CMASS/BMASS)
         CALL NEWTON1(T,AMB,AMC,NLOOP,NF4)
      ENDIF

c--- evaluate strong coupling at scale q
      IF (Q  .LT. BMASS) THEN
           IF (Q  .LT. CMASS) THEN
             T=2D0*DLOG(Q/CMASS)
             CALL NEWTON1(T,AMC,AS_OUT,NLOOP,NF3)
           ELSE
             T=2D0*DLOG(Q/BMASS)
             CALL NEWTON1(T,AMB,AS_OUT,NLOOP,NF4)
           ENDIF
      ELSE
      T=2D0*DLOG(Q/ZMASS)
      CALL NEWTON1(T,AMZ,AS_OUT,NLOOP,NF5)
      ENDIF
      ALPHAS=AS_OUT
      RETURN
      END




      subroutine lambda5qcd(lam,iset)
c this subroutine gives Lambda for five flavors for each set
      include 'nlooprun.f'
      character *36 pdfstring
      integer nset
      real*8 lambda5(0:92)
      real*8 lam
      common/prefix/nset,prefix
      common/pdfstring/pdfstring

      data lambda5/0.202d0,0.d0,0.181d0,0.1677d0,
     #             0.220d0,0.220d0,0.220d0,0.164d0,0.288d0,
     #             0.132d0,0.132d0,0.132d0,0.0d0,0.0d0,
     #             0.226d0,0.226d0,0.146d0,0.226d0,0.226d0,
     #             0.0d0,0.0d0,0.226d0,0.226d0,0.0d0,
     #             0.122d0,0.140d0,0.d0,0.d0,0.d0,0.d0,0.22d0,
     #             0.22d0,0.187d0,0.187d0,0.187d0,0.187d0,
     #             0.187d0,0.132d0,0d0,0d0,0d0,0.239d0,0.d0,
     #             0.d0,0.d0,0.181d0,0.d0,0.d0,0.d0,0.1665d0,
     #             0.226d0,0.226d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,
     #             0d0,0d0,0.2485d0,0.1795d0,0d0,0d0,0d0,0d0,0d0,
     #             0d0,0d0,0d0,0.260d0,0.1935d0,
     #             0d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,0d0,
     #             0d0,0d0,0d0,0d0,0d0,0.255d0,0.2551d0,0.1977d0/


!      write (6,*)
      write (6,*)'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC'
      write (6,*)'C                                                   C'
      write (6,*)'C      HRes running with native PDFs routines       C'
      write (6,*)'C                                                   C'
!      write (6,*)'CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC'
!      write (6,*)
!      write (6,49)'   pdf=',iset,', nset=',nset,', nloop=',nlooprun
!  49  format(A,I4,A,I3,A,I3)

      call cstring(pdfstring)


      write(6,*)'CCCCCCCCCC      Parton Distributions     CCCCCCCCCCCC'
      write(6,*)'C                                                   C'
      write(6,*)'C       ', pdfstring,'        C'
      write(6,*)'C                                                   C'

      lam=lambda5(iset)
      return
      end

      double precision function alphasNEW(nq2)
c.....N.B.:Here you have in output alpha_s/(4*Pi)!!!!!
      implicit none !real*8(a-h,o-z)
      double precision nq2,qq,t,xlt,alfa,b0,b1,b2,Pi,xlam,anf

      include 'nlooprun.f'
      COMMON/xlam/xlam
      real *8 qt,qt2,xtau,q,q2,shad,sroot,mur,mur2,muf,muf2
      common/somescale/qt,qt2,xtau,q,q2,shad,sroot,mur,mur2,muf,muf2
      data pi/3.1415926536d0/
      qq=sqrt(nq2)
      if(qq.lt.xlam .or. nq2.lt.0d0)then
       write(*,*)'ERROR: wrong argument in alphasNEW'
       stop
      endif
      anf=5d0
      b0=11D0-2D0/3D0*anf
      b1=51D0-19D0/3D0*anf
      b2=2857D0-5033D0/9D0*anf+325D0/27D0*anf**2

      t = 2.d0 * log(qq/xlam )
      xlt =log(t)
      if (nlooprun.eq.1) then
      alfa = 4d0*Pi/(b0 * t)
      elseif (nlooprun.eq.2) then
      alfa = 4d0*Pi/(b0 * t)*(1D0-2D0*b1/b0**2 * xlt/t)
      elseif (nlooprun.eq.3) then
      alfa = 4d0*Pi/(b0 * t)*(1D0-2D0*b1/b0**2 * xlt/t
     #+4D0*b1**2/b0**4/t**2*((xlt-0.5D0)**2+b2*b0/8D0/b1**2-5D0/4D0))
      endif
      alphasNEW=alfa/4/Pi
      return
      end
